@@ -1,14 +1,14 @@
-;; gpr-query.el --- Minor mode for navigating sources using gpr_query  -*- lexical-binding:t -*-
+;;; gpr-query.el --- Minor mode for navigating sources using gpr_query  -*- lexical-binding:t -*-
 ;;
 ;; gpr-query supports Ada and any gcc language that supports the
 ;; AdaCore -fdump-xref switch (which includes C, C++).
 ;;
-;; Copyright (C) 2013 - 2022  Free Software Foundation, Inc.
+;; Copyright (C) 2013 - 2023  Free Software Foundation, Inc.
 
 ;; Author: Stephen Leake <stephen_leake@member.fsf.org>
 ;; Maintainer: Stephen Leake <stephen_leake@member.fsf.org>
-;; Version: 1.0.1
-;; package-requires: ((emacs "25.3") (wisi "4.1") (gnat-compiler "1.0"))
+;; Version: 1.0.2
+;; package-requires: ((emacs "25.3") (wisi "4.2.0") (gnat-compiler "1.0.1"))
 
 ;; This file is part of GNU Emacs.
 
@@ -23,7 +23,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Usage:
 ;;
@@ -87,7 +87,7 @@ Must match gpr_query.adb Version.")
 
   symbol-locs ;; alist completion table, with locations; see gpr-query--symbols-filter
   symbols ;; symbols completion table.
-  symbols-valid ;; t when symbols valid
+  symbols-valid ;; nil initially, 'waiting when getting symbols, t when symbols valid
   symbols-count-total
   )
 
@@ -101,7 +101,7 @@ Must match gpr_query.adb Version.")
   "Regexp matching gpr_query prompt; indicates previous command is complete.")
 
 (defvar gpr-query--debug nil
-  "When non-nil, dispay debug messages.")
+  "When non-nil, display debug messages.")
 
 (defvar-local gpr-query--local-session nil
   "Buffer-local in gpr-query process buffer; the current session.")
@@ -173,6 +173,8 @@ Must match gpr_query.adb Version.")
   (let* ((count (line-number-at-pos (point)))
 	 (percent (/ (* 100 count) (gpr-query--session-symbols-count-total gpr-query--local-session))))
     (setq gpr-query--symbols-progress (format "%d%%" percent))
+    (when gpr-query--debug
+      (message "gpr-query update-progress %s" gpr-query--symbols-progress))
     ))
 
 (defun gpr-query--symbols-filter (process text)
@@ -256,8 +258,8 @@ Must match gpr_query.adb Version.")
 	(forward-line -1)
 	(when (re-search-forward gpr-query-prompt (point-max) t)
 	  (cond
-	   ((null (gpr-query--session-symbols gpr-query--local-session))
-	    ;; startup complete; get symbols
+	   ((null (gpr-query--session-symbols-count-total gpr-query--local-session))
+	    ;; startup complete; waiting for symbol count
 	    (when gpr-query--debug
 	      (message "gpr-query symbols: startup complete"))
 	     (gpr-query--check-startup)
@@ -265,10 +267,13 @@ Must match gpr_query.adb Version.")
 	     (set-marker (process-mark process) (point-min))
 	     (setf (gpr-query--session-symbols-count-total gpr-query--local-session) nil)
 	     (process-send-string process "complete \"\"\n")
-	     (setq gpr-query--symbols-progress "0%"))
+	     (setq gpr-query--symbols-progress "0%")
+	     (setf (gpr-query--session-symbols-valid gpr-query--local-session) 'waiting))
 
 	   (t
 	    ;; All symbols received; done
+	    (when gpr-query--debug
+	      (message "gpr-query symbols: all symbols received"))
 	    (setf (gpr-query--session-symbols-valid gpr-query--local-session) t)
 	    (set-process-filter process nil)
 	    (process-send-string process "exit\n"))
@@ -300,6 +305,7 @@ Must match gpr_query.adb Version.")
 	(with-current-buffer buffer
 	  (setq default-directory (file-name-directory gpr-file))
 	  (compilation-mode) ;; kills all local variables, requires default-directory
+	  (buffer-disable-undo)
 	  (setq gpr-query--local-session session)
 	  (setq buffer-read-only nil))))
 
@@ -376,6 +382,9 @@ If NO-SYMBOLS is non-nil, don't create the symbols process."
 (defun gpr-query-session-wait (session command-type)
   "Wait for the current COMMAND-TYPE command to complete.
 COMMAND-TYPE is one of xref or symbols."
+  (when gpr-query--debug
+    (message "gpr-query-session-wait %s" command-type))
+
   (when (and
 	 (eq command-type 'symbols)
 	 (null (gpr-query--session-symbols-process session)))
@@ -403,14 +412,14 @@ COMMAND-TYPE is one of xref or symbols."
 	 ;; The process filter is reading symbols text in the process
 	 ;; buffer; don't move point or otherwise modify the buffer.
 	 (cond
-	  ((gpr-query--session-symbols-valid session)
+	  ((eq (gpr-query--session-symbols-valid session) t)
 	   (setq done t))
 
 	  (t
 	   (message "gpr-query receiving symbols %s" gpr-query--symbols-progress))))
 
 	(xref
-	 (message (concat "running gpr_query ..." (make-string wait-count ?.)))
+	 (message "running gpr_query xref ... %s" (make-string wait-count ?.))
 
 	 ;; process output is inserted before point, so move back over it to search it
 	 (with-current-buffer (process-buffer process)
@@ -644,39 +653,6 @@ with compilation-error-regexp-alist set to COMP-ERR."
   (+ (abs (- found-col col))
      (* (abs (- found-line line)) 250)))
 
-(defvar gpr-query-map
-  (let ((map (make-sparse-keymap)))
-    ;; C-c C-i prefix for gpr-query minor mode
-
-    (define-key map "\C-c\C-i\C-d" 'gpr-query-goto-declaration)
-    (define-key map "\C-c\C-i\C-q" 'gpr-query-refresh)
-    (define-key map "\C-c\C-i\C-r" 'gpr-query-show-references)
-    ;; IMPROVEME: (define-key map "\C-c\M-d" 'gpr-query-parents)
-    ;; IMPROVEME: overriding
-    map
-  )  "Local keymap used for gpr query minor mode.")
-
-(defvar gpr-query-menu (make-sparse-keymap "gpr-query"))
-(easy-menu-define gpr-query-menu gpr-query-map "Menu keymap for gpr-query minor mode"
-  '("gpr-query"
-    ["Next xref"                     next-error 		   t]
-    ["Goto declaration/body"         xref-find-definitions 	   t]
-    ["Show parent declarations"      wisi-show-declaration-parents t]
-    ["Show references"               wisi-show-references 	   t]
-    ["Show overriding"               wisi-show-overriding 	   t]
-    ["Show overridden"               wisi-show-overridden 	   t]
-    ["Refresh cross reference cache" wisi-refresh-prj-cache 	   t]
-    ))
-
-(define-minor-mode gpr-query
-  "Minor mode for navigating sources using GNAT cross reference tool.
-Enable mode if ARG is positive."
-  :initial-value t
-  :lighter       " gpr-query"   ;; mode line
-
-  ;; just enable the menu and keymap
-  )
-
 (defun gpr-query--normalize-filename (file)
   "Convert FILE from native format to Emacs standard.
 FILE is from gpr-query."
@@ -703,9 +679,10 @@ FILE is from gpr-query."
   )
 
 ;;;###autoload
-(cl-defun create-gpr_query-xref (&key gpr-file)
+(cl-defun create-gpr_query-xref ()
   ;; See note on `create-ada-prj' for why this is not a defalias.
-  (make-gpr-query-xref :gpr-file gpr-file))
+  ;; The gpr file is set later by parsing the project file.
+  (make-gpr-query-xref))
 
 (cl-defmethod wisi-xref-parse-one ((xref gpr-query-xref) project name value)
   (wisi-compiler-parse-one xref project name value))
@@ -1094,4 +1071,4 @@ FILE is from gpr-query."
 	     (cons 'gpr-query-ident-file gpr-query-ident-file-regexp-alist))
 
 (provide 'gpr-query)
-;;; end of file
+;;; gpr-query.el ends here
